﻿using System;
using System.Net;
using NUnit.Framework;

namespace Josso.Agent.Lib.Tests
{
    [TestFixture]
    public class ZjsApplicationAgentTests
    {
        private ZjsApplicationAgent _agent;

        [SetUp]
        public void Setup()
        {
            _agent = new ZjsApplicationAgent(ZjsAgentTests.FAKE_GATEWAY, true);
            _agent.Josso = new ZjsAgentTests.ZjsMockedJosso();
        }

        /// <summary>
        /// Test the create web request method.
        /// </summary>
        [Test, Category("SMALL")]
        public void TestCreateWebRequest()
        {
            Uri uri = new Uri("https://url");
            HttpWebRequest request = _agent.CreateWebRequest(uri, ZjsAgentTests.USERNAME, null);
            Assert.AreEqual(1, request.CookieContainer.Count);

            CookieCollection cookies = request.CookieContainer.GetCookies(uri);
            Cookie cookie = cookies[ZjsBaseAgent.COOKIE_NAME];
            Assert.IsNotNull(cookie);
            Assert.AreEqual(ZjsAgentTests.ZjsMockedJosso.SSO_SESSION, cookie.Value);
        }

        /// <summary>
        /// Test the identify method.
        /// </summary>
        [Test, Category("SMALL")]
        public void TestIdentifyUser()
        {
            string session;
            bool identified = _agent.IdentifyUser(ZjsAgentTests.USERNAME, null, out session);
            Assert.IsTrue(identified);
            Assert.AreEqual(ZjsAgentTests.ZjsMockedJosso.SSO_SESSION, session);
        }

        /// <summary>
        /// Test to reset the cookie.
        /// </summary>
        [Test, Category("SMALL")]
        public void TestResetCookie()
        {
            ZjsMockedJossoForResetCookie josso = new ZjsMockedJossoForResetCookie();
            _agent.Josso = josso;

            Uri uri = new Uri("https://url");
            _agent.CreateWebRequest(uri, ZjsAgentTests.USERNAME, null);
            Assert.AreEqual(1, josso.CountIdentifyUserCall,
                            "It requires one user identification to build the request");

            _agent.CreateWebRequest(uri, ZjsAgentTests.USERNAME, null);
            Assert.AreEqual(1, josso.CountIdentifyUserCall,
                            "It should have used the internally saved cookie and not identify the user again");

            _agent.ResetCookie();
            _agent.CreateWebRequest(uri, ZjsAgentTests.USERNAME, null);
            Assert.AreEqual(2, josso.CountIdentifyUserCall,
                            "We have reset the cookie, it requires now to identify the user"
                            + " again in order to build the request");
        }

        private class ZjsMockedJossoForResetCookie : ZjsAgentTests.ZjsMockedJosso
        {
            public int CountIdentifyUserCall { get; private set; }

            public override bool IdentifyUser(string user, string password, out string ssoSession)
            {
                CountIdentifyUserCall++;
                return base.IdentifyUser(user, password, out ssoSession);
            }
        }
    }
}